/*

   Convert DSV (delimiter separated values) file to .dbf
	
	
	TODO:
	
	- resolve real type of character recorded values,
	  use these type into structure,
	  make required conversion while data transfer from txt to dbf
	  
	- add to TOOLBAR:
	    text file info
        browse text file
        edit structure	  
		dbf file info
        browse dbf
        exit
		
    - add progress bar at least to time consuming process 
	  ( FE Struct Analyzer, building from text etc for long files )
	
	- allow files built by program (.stu, .dbf) can had other than default names
	
	- warn user before overwriting existing files (built by program (.stu, .dbf) )
	
	
*/

#include "DSV2DBF.ch"

PROC Main()

    SET DATE GERM
    SET CENT ON
	
    _HMG_IsValidInProgres := .F.
    cDSVFNam	:= ""  	// delimiter separated values file name
    cDBFFNam	:= ""  	// DBF file name
	cStruFNam   := ""
	nFilHandle  := 0
	lFileOpen   := .F.
	lTFileOK    := .F.	
	aFldNams    := {}
	nFieldCo    := 0
    aStruArry   := {}
	   
    DEFINE WINDOW frmDSV2DBF_Main ;
        AT      0, 0 ;
        WIDTH   600 ;
        HEIGHT  400 ;
        TITLE   "Convert a DSV file to .DBF" ;
        ON INIT InitProg() ;
		ON RELEASE CloseSrcFile() ;
        MAIN 
      
        ON KEY ESCAPE ACTION ThisWindow.Release
		
	    DEFINE STATUSBAR FONT 'Verdana' SIZE 8   
		    STATUSITEM "" WIDTH 100
		    DATE          WIDTH 100
		    CLOCK         WIDTH 100
        END STATUSBAR
     
        DEFINE BUTTON btnOpen
           	ROW    5
            COL    5
            WIDTH  24
            HEIGHT 24		 
		    PICTURE  "bmpOpen"
			TOOLTIP  "Open source file"
            ONCLICK  OpenSrcFile()
        END BUTTON // btnOpen
		
        DEFINE BUTTON btnClose
           	ROW    5
            COL    32
            WIDTH  24
            HEIGHT 24
		    PICTURE  "bmpClose"
			TOOLTIP  "Close source file"
            ONCLICK  CloseSrcFile()
        END BUTTON // btnClose		
	  		
   END WINDOW // frmDSV2DBF_Main

   frmDSV2DBF_Main.Center

   frmDSV2DBF_Main.Activate
   
RETURN // Main()

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

PROC InitProg( )
    OpenSrcFile()
RETU // InitProg()

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

PROC OpenSrcFile()

    cDSVFNam := GetFile ( { {"DSV Files",  SrcFFilt },;
                              {"XLS files",  "*.XLS"  },; 
							  {"Text files", "*.TXT"  } },;  // acFilter
							"Delimiter separated values files",; // cTitle
							GetCurrentFolder(),;  // cDefaultPath
							.F.,;  // lMultiSelect
							.F. )  // lNoChangeDir
				

	IF !EMPTY( cDSVFNam )
        HB_FUSE()          // Close text file if any
	    nFilHandle := 0
        aFldNams   := {}
	    nFieldCo   := 0
        	
	    IF FILE( cDSVFNam )
	       nFilHandle := HB_FUse( cDSVFNam )
	       lFileOpen  := nFilHandle > 0	   
	    ENDI FILE( cDSVFNam )
	    
	    IF lFileOpen
	       IF HB_FLastRec() > 0
	          aFldNams := HB_ATOKENS( HB_FReadLn(), cFldDelim )
	    	  nFieldCo := LEN( aFldNams )
	       ELSE	  
        	   MsgsTOP( cDSVFNam + " file is empty")
	       ENDI
	    ELSE
	       MsgsTOP( "Couldn't open " + cDSVFNam, "Open Error")
	    ENDIF lFileOpen
	    
	    lTFileOK := lFileOpen .AND. ! EMPTY( aFldNams )
		
		IF lTFileOK
   	        frmDSV2DBF_Main.STATUSBAR.Item(1) := cDSVFNam
		    CheckStruFile()
		    IF !EMPTY( aStruArry )
		        BuildDBF()
		    ENDIF
	    ELSE
	        cDSVFNam := ""
	    ENDIF lTFileOK
		
	ENDIF !EMPTY( cDSVFNam )
		
RETU // OpenSrcFile()

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

PROC CloseSrcFile()

    IF lFileOpen
        HB_FUSE()
	    frmDSV2DBF_Main.STATUSBAR.Item(1) := ""
	ENDI  
	
    cDSVFNam	:= ""  	
    cDBFFNam	:= ""  	
	cStruFNam   := ""
	nFilHandle  := 0
	lFileOpen   := .F.
	lTFileOK    := .F.	
	aFldNams    := {}
	nFieldCo    := 0
    aStruArry   := {}
	
RETU // CloseSrcFile()

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

FUNC CheckStruFile()

	LOCA cStruText,;
		 nFldNum,;
		 a1Field := {},;
		 nErrCod,;
         lStructoK := .T.		 

    cStruFNam := LEFT( cDSVFNam, HB_RAT(  ".", cDSVFNam) ) + StruFExt
	
	aStruArry := {}
	
    IF FILE( cStruFNam )
	    IF FILESIZE( cStruFNam ) >= nFieldCo * nMinStuLLen - 2   
		
    	    cStruText := HB_MEMOREAD( cStruFNam )    	   
	
            aStruArry := Txt2Arr2D( cStruText, CRLF, " " )
		
            FOR nFldNum := 1 TO LEN( aStruArry )
                a1Field := aStruArry[ nFldNum ]
				aStruArry[ nFldNum, 3 ] := VAL( aStruArry[ nFldNum, 3 ] )  // Field width
				aStruArry[ nFldNum, 4 ] := VAL( aStruArry[ nFldNum, 4 ] )  // Field decimal 
			NEXT	
			
            IF LEN( aStruArry ) # nFieldCo
				lStructoK := .F.
				nErrCod := 1
			ELSE 
			    FOR nFldNum := 1 TO LEN( aStruArry )
				    IF LEN( aStruArry[ nFldNum ] ) # 4
                        lStructoK := .F.
						nErrCod := 2
						EXIT // FOR
				    ENDIF		
				NEXT
            ENDIF 			
		ELSE	
            lStructoK := .F.	
			nErrCod := 3
		ENDIF		
	ELSE
        lStructoK := .F.
		nErrCod := 4
	ENDIF

    IF !lStructoK
	    IF MsgYesNo( {  "Ooops !",;
		                CRLF2,;
                        "structure file",; 
		                CRLF2,;
						cStruFNam,; 
		                CRLF2,;
						"not found",; 
				        CRLF2,;
			  	        "or invalid or damaged ! ( Error Code is: " + NTOC( nErrCod ) + " )",;
				        CRLF2,;
				        "Do you want build a new ?";
			         }, "Structue ERROR !" )					 
            BuildStruct() 
			lStructoK := !EMPTY( aStruArry )
		ENDIF		
	ENDIF
	
    IF ! lStructoK
	    CloseSrcFile()		
	ENDIF	
	
RETU lStructoK // CheckStruFile()

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

PROC BuildStruct()			

    LOCA c1FldVal   := "",;
	     n1FldNum   := 0,;
         aFields    := {},;
         aFldWidths := {}
		 
	WaitWindow( "Analyzing ...", .T. )	
				
    aStruArry := {}
	
    HB_FGoTop()
	
	WHIL !HB_FEOF()
        aFields := HB_ATOKENS( HB_FReadLn(), cFldDelim )
	    FOR n1FldNum := 1 TO LEN( aFields	)   
	        c1FldVal := aFields[ n1FldNum ]
	        IF LEN( aFldWidths ) < n1FldNum
		       AADD( aFldWidths, 0 ) 
		    ENDIF  
			aFldWidths[ n1FldNum ] := MAX( aFldWidths[ n1FldNum ], LEN( c1FldVal ) )						
	    NEXT
		HB_FSKIP()
	ENDD   
	
    IF LEN( aFldWidths ) # LEN( aFldNams )
	    MsgStop( { "Ooops !",;
				   CRLF2,;
                   "we have a problem",; 
				   CRLF2,;
			  	   "Field name count not equal to field count",;
				   CRLF2,;
				   "Please check content of your source file (cDSVFNam) ";
				}, "Structue ERROR !" )
		CloseSrcFile()		
	ELSE
        AEVAL( aFldWidths, { | n1, i1 | AADD( aStruArry, { aFldNams[ i1 ], "C", n1, 0 } ) } )	// see TODO
		
	    HB_MEMOWRIT( cStruFNam, Arr2D2Txt( aStruArry, CRLF, " " ) )  // Automatic save	

        MsgInfo( { "Structure built and saved with name: " ,;		
		           CRLF2,;
				   cStruFNam } )
				   
	ENDIF

	WaitWindow()
	
RETU // BuildStruct()			

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._
	
PROC BuildDBF()	

    LOCA cdBasFNam := LEFT( cDSVFNam, HB_RAT(  ".", cDSVFNam) ) + dBasFExt,;
	     aFields,;
		 n1FldNum
		 
	* TODO: IF FILE( cdBasFNam ) ... overwrite or not ...

	DBCREATE( cdBasFNam, aStruArry )
	
	IF FILE( cdBasFNam )
	
	    MsgInfo( { "New DBF file built with name",;
		           CRLF2,;
				   cdBasFNam } )
	    USE ( cdBasFNam )

        IF USED()		
            HB_FGoTop()
		
	        HB_FSKIP()   // Header ( field names ) line / record
		
	        WHIL !HB_FEOF()
				aFields := HB_ATOKENS( HB_FReadLn(), cFldDelim )
				DBAPPEND()
				FOR n1FldNum := 1 TO LEN( aFields	)   
					FIELDPUT( n1FldNum, aFields[ n1FldNum ] )
				NEXT		    
				HB_FSKIP()
			ENDD   			
    	    MsgInfo( "New DBF file filled with text file data", "Success" )			
	    ELSE
            MsgStop( "Couln't USE newly built DBF file !", "DB File USE error")		
		ENDIF   
		
	ENDIF FILE( cdBasFNam )
RETU // BuildDBF()

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

